function [Hs, Hz] = Hs_linear(filterOrder, fp_rel, transBw_rel, varargin)
%HS_LINEAR   linear phase lowpass filter design.
%	[Hs, Hz] = HS_LINEAR(filterOrder, fp_rel, transBw_rel) returns the
%	structures Hs and Hz describing the transfer functions of a linear 
%	phase approximation of the ideal lowpass filter at the relative cutoff 
%   frequency f/fs and a transition bandwidth B/fs. 
%	The structures Hs/Hz are organized as follows:
%		Hs/z.poly_fs/z     -- the coefficients of the numerator function
%		Hs/z.poly_gs/z     -- the coefficients of the denominator function
%		Hs/z.ident         -- a string, describing the filter
%		Hs/z.roots_fs/z    -- the roots of the numerator 
%		Hs/z.roots_gs/z    -- the roots of the denominator
%	where poly_fs and poly_gs are vectors of coefficients in descending 
%	powers of s.
%   The filterOrder parameter represents the number of derived
%   multipliers and not the order of the transfer function.
%   The relative frequencies and bandwidths must be specified in z-domain
%   value range [0, 0.5[.
%
%	[Hs, Hz] = HS_LINEAR(filterOrder, fp_rel, transBw_rel, 'DesignMethod', 'collocation')
%   additionally specifies the design method.
%
%	[Hs, Hz] = HS_LINEAR(filterOrder, fp_rel, transBw_rel, 'OrderReduction', n)
%   performs a reduction of order n.
%
%	[Hs, Hz] = HS_LINEAR(filterOrder, fp_rel, transBw_rel, 'IterativeOptimisation', 1)
%   performs an iterative optimisation of the phase response.
%
%	See also HS_BUTTER, HS_CHEBY, HS_INVCHEBY, HS_CAUER, HS_CAUER_BIREC, HS_VLACH.
%
% (c) H. G. Brachtendorf, C. Zeintl
%     University of Applied Sciences Upper Austria, Hagenberg 2017

% parse input arguments
p = inputParser;
p.CaseSensitive = false;

addRequired (p, 'filterOrder');
addRequired (p, 'fp_rel');
addRequired (p, 'transBw_rel');
addParameter(p, 'DesignMethod', 'collocation');
addParameter(p, 'OrderReduction', 0);
addParameter(p, 'IterativeOptimisation', 0);

parse(p, filterOrder, fp_rel, transBw_rel, varargin{:});

designMethod = p.Results.DesignMethod;
orderReduction = p.Results.OrderReduction;
iterativeOptimization = p.Results.IterativeOptimisation;

% evaluate range of input arguments
assert(rem(filterOrder,2)==1, 'Hs_linear:filterOrder', 'filterOrder must be an odd, poitive number');
assert((orderReduction >= 0) && (orderReduction < filterOrder), 'Hs_linear:orderReduction', 'orderReduction must be smaller than filterOrder');
assert(fp_rel >= 0, 'Hs_linear:fp_rel', 'passband edge frequency must be positive');
assert(transBw_rel >= 0, 'Hs_linear:transition band', 'transition bandwidth must be positive');
%assert((fp_rel + transBw_rel) < 0.5, 'Hs_linear:transition band', 'transition bandwidth exceeds fs/2');
if (sum(strcmp({'galerkin', 'collocation'}, designMethod)) == 0)
    assert(false, 'Hs_linear:designMethod', 'designMethod must be one of the following ''galerkin'' | ''collocation''');
end

Omega_D = fp_rel * 2 * pi;
Omega_S = (fp_rel + transBw_rel) * 2 * pi;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Auswahl der Frequenzen Omega fuer Collocation
if strcmp(designMethod, 'collocation')
    CollocationPoints = 71; %223;  71
    assert(CollocationPoints >= filterOrder);

    CollocationPoints1 = floor(CollocationPoints/4);
    %CollocationPoints2 = ceil(CollocationPoints/10);
    CollocationPoints2 = 5;
    CollocationPoints3 = CollocationPoints - CollocationPoints1 - CollocationPoints2;

    %Randpunkte ausnehmen
    Omega1 = linspace(0,Omega_D, CollocationPoints1 + 1); 
    Omega1 = Omega1(1:length(Omega1)-1); %Durchlass
    Delta_Omega = Omega_S-Omega_D; assert(Delta_Omega > 0);
    Omega2 = linspace(Omega_D,Omega_D + Delta_Omega, CollocationPoints2 + 2); %Uebergang
    Omega2 = Omega2(2:length(Omega2)-1);
    Omega3 = linspace(Omega_S,pi, CollocationPoints3 + 1);
    Omega3 = Omega3(1:length(Omega3)-1); %Sperrbereich
    Omega  = [Omega1,Omega2,Omega3];
else
    Omega = linspace(0,pi,2^10);
end

%% weighting function
if strcmp(designMethod, 'collocation')
    weight_pass = 10; weight_stop1 = 10; weight_stop2 = 100;
    weight_DC  = 1000; %numerische Stabilisierung
else
    weight_pass = 1; weight_stop1 = 1000; weight_stop2 = 100;
    weight_DC  = 1000; %numerische Stabilisierung   
end

if strcmp(designMethod, 'collocation')
    W = ones(1,CollocationPoints); %Hintergrundgewichtung

%    stopregion1 = round(1.1*CollocationPoints1+CollocationPoints2+1);
%    stopregion2 = round(1.7*CollocationPoints1+CollocationPoints2+1);

    stopregion1 = length([Omega1 Omega2]) + 1;
    stopregion2 = length([Omega1 Omega2]) + 20;

    W(1:round(1.0*CollocationPoints1)) = weight_pass; %passband 
    W(stopregion1:stopregion2-1) = weight_stop1; %stopband region 1
    W(stopregion2:CollocationPoints) = weight_stop2; %stopband region 2
    W(1:1) = weight_DC;
else
    W = ones(1,length(Omega)); %Hintergrundgewichtung
    ind = Omega <= Omega_D; W(ind) = weight_pass; W(1) = weight_DC;
    ind = Omega >= Omega_S; W(ind) = weight_stop1;
    ind = Omega >= 1.2*Omega_S; W(ind) = weight_stop2;

    %stopregion1 = find(Omega > Omega_S, 1 );
    %stopregion2 = find(Omega > Omega_D, 1 ); %muss ueberarbeitet werden
end

%% Referenzzweig S2 ein Grad niedriger
%  falls orderReduction = 0 einfaches Delay Glied

M = filterOrder-1;
Phi_S2   = (M)*Omega; %delay filterOrder-1

if orderReduction == 0
    if strcmp(designMethod, 'collocation')
        [bd_S2,ad_S2] = AllpassCollocation(M,Omega,Phi_S2,W,[1,1]);
    else
        [bd_S2,ad_S2] = AllpassGalerkin(M,Omega,Phi_S2,W);
    end
else % nur der Durchlassbereich ist relevant
    if strcmp(designMethod, 'collocation')
        [bd_S2,ad_S2] = AllpassCollocation(M-orderReduction,Omega(1:CollocationPoints1), ...
                        Phi_S2(1:CollocationPoints1),W(1:CollocationPoints1),[1,1]);
    else
        error('Galerkin not implemented');
    end
end

%transfer function
%[Hd_S2]  = freqz(bd_S2,ad_S2,OmegaPlot);
%Gd_S2    = grpdelay(bd_S2,ad_S2,OmegaPlot);
Phi_S2cal = unwrap(angle((freqz(bd_S2,ad_S2,Omega))));
Phi_S2cal = -Phi_S2cal;
%pzmap(bd_S2,ad_S2);

%% Zweig S1

% Allpass S1
if strcmp(designMethod, 'collocation')
    Phi_S1 = zeros(size(Phi_S2cal)); 
    Phi_S1(1:CollocationPoints1) = 2*(M)*Omega1 - Phi_S2cal(1:CollocationPoints1); %Korrektur ripple
    %slope = linspace(0,pi,CollocationPoints2+2); slope =
    %slope(2:CollocationPoints2+1); % Uebergang
    slope = pi/2 + atan(linspace(-5,5,CollocationPoints2+2)); slope = slope(2:CollocationPoints2+1); % Uebergang
    Phi_S1(CollocationPoints1+1:CollocationPoints1+CollocationPoints2) = ...
        Phi_S2cal(CollocationPoints1+1:CollocationPoints1+CollocationPoints2) + slope;
    Phi_S1(CollocationPoints1+CollocationPoints2+1:CollocationPoints) = ... % Sperrbereich
       Phi_S2cal(CollocationPoints1+CollocationPoints2+1:CollocationPoints) + pi;
else
    Phi_S1 = zeros(size(Omega)); 
    ind = find(Omega <= Omega_D); %Durchlass
    Phi_S1(ind) = 2*(M)*Omega(ind) - Phi_S2cal(ind); %Korrektur ripple
    ind = find(Omega > Omega_D & Omega < Omega_S); %Uebergang
    slope = pi/2 + atan(linspace(-100,100,length(ind)+2)); slope = slope(2:length(ind)+1); % Uebergang
    Phi_S1(ind) =  M*Omega(ind) + slope;
    ind = find(Omega >= Omega_S); % Sperrbereich
    Phi_S1(ind) = M*Omega(ind) + pi;
end


if orderReduction == 0
    if strcmp(designMethod, 'collocation')
        [bd_S1,ad_S1] = AllpassCollocation(filterOrder,Omega,Phi_S1,W,[1,1]);
    else
        [bd_S1,ad_S1] = AllpassGalerkin(filterOrder,Omega,Phi_S1,W);
    end
else
    if strcmp(designMethod, 'collocation')
        [bd_S1,ad_S1] = AllpassCollocation(filterOrder-orderReduction,Omega,Phi_S1,W,[1,1]);
    else
        error('Galerkin not implemented');
    end
end

%[Hd_S1]  = freqz(bd_S1,ad_S1,OmegaPlot);
%Gd_S1    = grpdelay(bd_S1,ad_S1,OmegaPlot);
Phi_S1cal = unwrap(angle((freqz(bd_S1,ad_S1,Omega))));
Phi_S1cal = -Phi_S1cal;

%error('break');

%% Referenzzweig S2 once again zur Reduktion des group delay ripples
% 1. Schritt einer iterativen Optimierung
if iterativeOptimization
    
    Phi_S2(1:floor(0.9*CollocationPoints1)) = 2*(M)*Omega1(1:floor(0.9*CollocationPoints1)) ...
        - Phi_S1cal(1:floor(0.9*CollocationPoints1)); %Korrektur ripple
    % slope = linspace(0,pi,CollocationPoints2+2); slope = slope(2:CollocationPoints2+1); % Uebergang
    % Phi_S2(CollocationPoints1+1:CollocationPoints1+CollocationPoints2) = ...
    %     (M)*Omega2;
    % %    2*(M)*Omega2 - Phi_S1cal(CollocationPoints1+1:CollocationPoints1+CollocationPoints2);
    Phi_S2(CollocationPoints1+CollocationPoints2+1:CollocationPoints) = ... % Sperrbereich
       Phi_S1cal(CollocationPoints1+CollocationPoints2+1:CollocationPoints) - pi;


    if orderReduction == 0
        if strcmp(designMethod, 'collocation')
            [bd_S2,ad_S2] = AllpassCollocation(M,Omega,Phi_S2,W,[1,1]);
        else
            [bd_S2,ad_S2] = AllpassGalerkin(M,Omega,Phi_S2,W);
        end
    else % nur der Durchlassbereich ist relevant
         if strcmp(designMethod, 'collocation')
            [bd_S2,ad_S2] = AllpassCollocation(M-orderReduction,Omega,Phi_S2,W);
         else
            error('Galerkin not implemented');
        end
    end

    %transfer function
    %[Hd_S2]  = freqz(bd_S2,ad_S2,OmegaPlot);
    %Gd_S2    = grpdelay(bd_S2,ad_S2,OmegaPlot);
    %Phi_S2cal = unwrap(angle((freqz(bd_S2,ad_S2,Omega))));
    %Phi_S2cal = -Phi_S2cal;

end

%% check stability criterion
% all roots of the denominator polynomials must lie within the unit circle
assert(all(abs(roots(ad_S1)) < 1));
assert(all(abs(roots(ad_S2)) < 1));

%% combine the transfer functions of reference and reflection branches
% S = 1/2 * (S1 + S2)
b_S = conv(ad_S1, bd_S2) + conv(ad_S2, bd_S1);
a_S = 2 * conv(ad_S1, ad_S2);

%% copy the results into transfer function structures
Hz.poly_fz  = b_S; % numerator polynomial coefficients
Hz.poly_gz  = a_S; % denominator polynomial coefficients
Hz.ident    = [ 'LP PROTOTYPE: ''linear'',' num2str(filterOrder) ',' num2str(0) ...
							',' num2str(fp_rel) ',' num2str(0) ];
Hz.roots_fz = roots(b_S);
Hz.roots_gz = roots(a_S);

Hz1.poly_fz  = bd_S1; % numerator polynomial coefficients
Hz1.poly_gz  = ad_S1; % denominator polynomial coefficients
Hz1.ident    = [ 'LP PROTOTYPE: ''linear'',' num2str(filterOrder) ',' num2str(0) ...
							',' num2str(fp_rel) ',' num2str(0) ];
Hz1.roots_fz = roots(bd_S1);
Hz1.roots_gz = roots(ad_S1);
Hz2.poly_fz  = bd_S2; % numerator polynomial coefficients
Hz2.poly_gz  = ad_S2; % denominator polynomial coefficients
Hz2.ident    = [ 'LP PROTOTYPE: ''linear'',' num2str(filterOrder) ',' num2str(0) ...
							',' num2str(fp_rel) ',' num2str(0) ];
Hz2.roots_fz = roots(bd_S2);
Hz2.roots_gz = roots(ad_S2);

%% convert to s-domain and return the results for s- and z-domain
Hs = Hz2Hs(Hz);
Hz = [Hz1 Hz2];
