function [Hs, Hz] = Hs_linearBP(filterOrder,bw_rel,transBw_rel,varargin)
%HS_LINEAR   linear phase lowpass filter design.
%	[Hs, Hz] = HS_LINEARBP(filterOrder, bw_rel, transBw_rel) returns the
%	structures Hs and Hz describing the transfer functions of a linear 
%	phase approximation of a bandpass filter which is symmetrical w.r.t.
%   to fs/4 at the relative bandwidth bw_rel.
%	The structures Hs/Hz are organized as follows:
%		Hs/z.poly_fs/z     -- the coefficients of the numerator function
%		Hs/z.poly_gs/z     -- the coefficients of the denominator function
%		Hs/z.ident         -- a string, describing the filter
%		Hs/z.roots_fs/z    -- the roots of the numerator 
%		Hs/z.roots_gs/z    -- the roots of the denominator
%	where poly_fs and poly_gs are vectors of coefficients in descending 
%	powers of s. 
%   The filterOrder parameter represents the number of derived
%   multipliers and not the order of the transfer function.
%   The relative frequencies and bandwidths must be specified in z-domain
%   value range [0, 0.5[.
%
%	[Hs, Hz] = HS_LINEARBP(filterOrder, bw_rel, transBw_rel, 'DesignMethod', 'collocation')
%   additionally specifies the design method.
%
%	[Hs, Hz] = HS_LINEARBP(filterOrder, bw_rel, transBw_rel, 'OrderReduction', n)
%   performs a reduction of order n.
%
%	[Hs, Hz] = HS_LINEARBP(filterOrder, bw_rel, transBw_rel, 'IterativeOptimisation', 1)
%   performs an iterative optimisation of the phase response.
%
%	See also HS_BUTTER, HS_CHEBY, HS_INVCHEBY, HS_CAUER, HS_CAUER_BIREC, HS_VLACH.
%
% (c) H. G. Brachtendorf, C. Zeintl
%     University of Applied Sciences Upper Austria, Hagenberg 2017

% parse input arguments
p = inputParser;
p.CaseSensitive = false;

addRequired (p, 'filterOrder');
addRequired (p, 'bw_rel');
addRequired (p, 'transBw_rel');
addParameter(p, 'DesignMethod', 'collocation');
addParameter(p, 'OrderReduction', 0);
addParameter(p, 'IterativeOptimisation', 0);

parse(p, filterOrder, bw_rel, transBw_rel, varargin{:});

designMethod = p.Results.DesignMethod;
orderReduction = p.Results.OrderReduction;
iterativeOptimization = p.Results.IterativeOptimisation;

% evaluate range of input arguments
assert(rem(filterOrder,2)==0, 'Hs_linearBP:filterOrder', 'filterOrder must be an even, poitive number');
assert((orderReduction >= 0) && (orderReduction < filterOrder), 'Hs_linear:orderReduction', 'orderReduction must be smaller than filterOrder');
assert(bw_rel > 0, 'Hs_linear:bandWidth', 'bandWidth must be positive');
assert(transBw_rel > 0, 'Hs_linear:transitionBandwidth', 'transitionBandwidth must be positive');
if (sum(strcmp({'galerkin', 'collocation'}, designMethod)) == 0)
    assert(false, 'Hs_linear:designMethod', 'designMethod must be one of the following ''galerkin'' | ''collocation''');
end

% sum of all bandwidths must be smaller than pi
%passTransitionBand = bw_rel + 2 * transBw_rel;
%assert(passTransitionBand < pi, 'Hs_linear:bandwith exceeded', 'bandwidths of passband and transition bands exceed spectrum');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% design LP with half the bandwidth's cutoff frequency and odd order
fp_rel_LP = bw_rel / 2;
filterOrderLP = filterOrder - 1;

[~, HzLP] = Hs_linear(filterOrderLP, fp_rel_LP, transBw_rel, 'DesignMethod', designMethod, ...
                    'OrderReduction', orderReduction, 'IterativeOptimisation', iterativeOptimization); 

% get the coefficient vectors of top and bottom branches of LP filter
ad_S1 = HzLP(1).poly_gz; bd_S1 = HzLP(1).poly_fz;
ad_S2 = HzLP(2).poly_gz; bd_S2 = HzLP(2).poly_fz;

% some params
%OmegaPlot = linspace(0,pi,512);
N         = length(ad_S1)-1; % filter length
M         = N-1;

%% lowpass to bandpass transform
% transform z^-1 -> -z^-2
ad_bp_S1 = (-1).^(0:N) .* ad_S1; ad_bp_S1 = kron(ad_bp_S1,[1 0]);
bd_bp_S1 = (-1).^(0:N) .* bd_S1; bd_bp_S1 = kron(bd_bp_S1,[1 0]);
ad_bp_S2 = (-1).^(0:M) .* ad_S2; ad_bp_S2 = kron(ad_bp_S2,[1 0]);
bd_bp_S2 = (-1).^(0:M) .* bd_S2; bd_bp_S2 = kron(bd_bp_S2,[1 0]);

%% check stability criterion
% all roots of the denominator polynomials must lie within the unit circle
assert(all(abs(roots(ad_bp_S1)) < 1));
assert(all(abs(roots(ad_bp_S2)) < 1));

%% combine the transfer functions of reference and reflection branches
% S = 1/2 * (S1 + S2)
b_S = conv(ad_bp_S1, bd_bp_S2) + conv(ad_bp_S2, bd_bp_S1);
a_S = 2 * conv(ad_bp_S1, ad_bp_S2);

%% copy the results into transfer function structures
Hz.poly_fz  = b_S; % numerator polynomial coefficients
Hz.poly_gz  = a_S; % denominator polynomial coefficients
Hz.ident    = [ 'BP PROTOTYPE: ''linear'',' num2str(filterOrder) ',' num2str(0.25) ...
							',' num2str(bw_rel) ',' num2str(0) ];
Hz.roots_fz = roots(b_S);
Hz.roots_gz = roots(a_S);

Hz1.poly_fz  = bd_bp_S1; % numerator polynomial coefficients
Hz1.poly_gz  = ad_bp_S1; % denominator polynomial coefficients
Hz1.ident    = [ 'BP PROTOTYPE: ''linear'',' num2str(filterOrder) ',' num2str(0.25) ...
							',' num2str(bw_rel) ',' num2str(0) ];
Hz1.roots_fz = roots(bd_bp_S1);
Hz1.roots_gz = roots(ad_bp_S1);
Hz2.poly_fz  = bd_bp_S2; % numerator polynomial coefficients
Hz2.poly_gz  = ad_bp_S2; % denominator polynomial coefficients
Hz2.ident    = [ 'BP PROTOTYPE: ''linear'',' num2str(filterOrder) ',' num2str(0.25) ...
							',' num2str(bw_rel) ',' num2str(0) ];
Hz2.roots_fz = roots(bd_bp_S2);
Hz2.roots_gz = roots(ad_bp_S2);

%% convert to s-domain and return the results for s- and z-domain
Hs = Hz2Hs(Hz);
Hz = [Hz1 Hz2];
