function [LWDF,Hz,Messages] = Hz2LWDF(Hz,figNo,showMessages,plotOptionsString)
%HS2LWDF	Calculates the coefficients of a Lattice Wave Digital Filter 
%	LWDF = HZ2LWDF(Hz) creates a structure LWDF given a transfer function Hz.
%	The structure Hz should be organized as follows:
%		Hz.poly_fz         -- the coefficients of the numerator function
%		Hz.poly_gz         -- the coefficients of the denominator function
%		Hz.ident           -- a string, describing the filter
%		Hz.roots_fz        -- the roots of the numerator 
%		Hz.roots_gz        -- the roots of the denominator
%	where poly_fz and poly_gz are vectors of coefficients in descending 
%	powers of z. Odd polynomials Hz.poly_gz result in low/high pass LWDFs,
%	even polynomials in bandpass/stop LWDFs.
%	The structure LWDF will contain the fields
%		LWDF.wdaCodes and
%		LWDF.gamma
%	LWDF.wdaCodes is an array of 2 strings, which describe the presence
%	and positions of the adaptors to be used. The following adaptor 
%	combinations are recognized
%		't' - a single delay element
%		's' - one 2-port and one delay element
%		'S' - one 2-port with two cascaded delay elements
%		'd' - two 2-ports with two delay elements
%		'D' - two 2-ports with two times two cascaded delay elements
%		'x' - only an interconnection in this slot
%	LWDF.gamma gives the coefficient values for the 2-ports.
%	The block diagram shown (default) can be used to see which adaptor 
%	corresponds to which coefficient.
%	
%	[...] = HZ2LWDF(Hz,figNo) can be used to control the block diagram plot. 
%	Use figNo = 0 if no output is wanted. When no figNo is specified, 
%	figure(1) will be used for plotting, otherwise figure(figNo). 
%
%	[LWDF,Hz,Messages] = HZ2LWDF(Hz,figNo,showMessages) can be used to
%	control the printing of error messages in the workspace window.
%	showMessages 1 acts as 'normal': errors are signalled, while 
%	showMessages 0 suppresses output and returns the error messages in the 
%	output string Messages.
%
%	[LWDF,Hz,Messages] = HZ2LWDF(Hz,figNo,showMessages,plotOptionsString)
%	To enable the output of an additional Hz plot, plotOptions can be entered
%	(as a string), which are passed to PLOTHZ. See PLOTHZ for an extensive
%	description of the options.
%
%	Examples:
%		% a 7th order lowpass filter (linear phase approximation method)
%		LWDF = Hz2LWDF(Hs_linear(7, fz2fs(0.15)), 1, 1, '1,2');
%
%	See also SHOWLWDF, LWDF2HZ, HS2LWDF.

% (c) H.G. Brachtendorf, C. Zeintl
%     University of Applied Sciences Upper Austria, Hagenberg 2017

if ~exist('showMessages', 'var') 
	showMessages = 1;
end
if ~exist('figNo', 'var') 
	figNo = 1;
end
% configuration of matlab function
ctol      = 1e-12;    % absolute value beneath which a coefficient is treated as zero


% extract roots and sort them
isLinear = strfind(Hz(1).ident, 'linear');
if isLinear
    % transfer function split up to top and bottom branch (linear phase)
    % just take roots from top branch of ladder structure
    roots_gz     = cplxpair( Hz(1).roots_gz );
    filterOrder  = length( Hz(1).roots_gz ) + length( Hz(2).roots_gz );
else
    % single transfer function in Hz
    assert(false, 'single transfer function not implemented yet!');
    roots_gz     = cplxpair( Hz.roots_gz );
    filterOrder  = length( Hz.roots_gz );
end

disp(roots_gz); % DBG

msgStr = '';
errStr = '';
%filterOrder    = length( roots_gz );
isLowPass      = ( rem(filterOrder,2) == 1 ); % number of roots indicates LP/HP or BP/BS

try
    
    % sort top row filter roots 
    rootsSorted = sort(roots_gz, 'ascend'); % sort by ascending absolute value
    rootsReal = rootsSorted(imag(rootsSorted) == 0);
    rootsImag = rootsSorted(imag(rootsSorted) ~= 0);
    [rootsZero, rootsReal, rootsImag] = lfSortRoots(roots_gz, ctol)
    %roots_gz = [rootsReal ; rootsImag]
    
    % count number of filter coefficients (multipliers), for initialization of result structure
    numBot = length( Hz(2).roots_gz ) - length( rootsZero );
    numTop = 0;
    idx = 1;
    errStr = 'unexpected number of roots with same magnitude!';
    while idx <= length(rootsReal)
        ctEl = lfGetSameMag(rootsReal, idx, ctol);
        assert((ctEl == 2) || (ctEl == 1), 'unexpected number of real roots with same magnitude!');
        numTop = numTop + 1;
        
        idx = idx + ctEl;
    end
    idx = 1;
    while idx <= length(rootsImag)
        ctEl = lfGetSameMag(rootsImag, idx, ctol);
        assert((ctEl == 2) || (ctEl == 4), 'unexpected number of imag roots with same magnitude!');
        numTop = numTop + 2;
        
        idx = idx + ctEl;
    end
    
    % initialize LWDF structure
    nSlices = ceil(numTop / 2);
    codes = char( 'x' * repmat( [1;1],1,nSlices) );
    gamma = NaN * ones(2, nSlices, 2);
    
    %top all pass sections (start with pure roots)
    idxSec = 1;
    idx = 1;
    errStr = 'polynomial value unexpectedly non zero!';
    while idx <= length(rootsReal)
        if (rem(length(rootsReal),2) ~= 0) && (idx == 1)
            % create 1st order section
            codes(1,idxSec) = 's';
            gamma(1,idxSec,1) = rootsReal(idx);
            idx = idx + 1;
        else
            if ((lfGetSameMag(rootsReal, idx, ctol) == 2) && (lfGetSameMag(rootsReal, idx+2, ctol) == 2))
                % create 4th order section
                codes(1,idxSec) = 'D';
                p = poly([rootsReal(idx) rootsReal(idx+1) rootsReal(idx+2) rootsReal(idx+3)]);
                % TODO assertion for elements 2 and 4 of polynomial (!!!must be zero!!!)
                gamma(1,idxSec,1) = -p(5);
                gamma(2,idxSec,1) = -p(3)/(1+p(5));
                idx = idx + 4;
            elseif (lfGetSameMag(rootsReal, idx, ctol) == 2)
                % create 2nd order section (one coefficient)
                codes(1,idxSec) = 'S';
                p = poly([rootsReal(idx) rootsReal(idx+1)]);
                gamma(1,idxSec,1) = -p(3);
                gamma(2,idxSec,1) = -p(2)/(1+p(3));
                idx = idx + 2;
            else
                % create 2nd order section (two coefficients)
                codes(1,idxSec) = 'd';
                p = poly([rootsReal(idx) rootsReal(idx+1)]);
                gamma(1,idxSec,1) = -p(3);
                gamma(2,idxSec,1) = -p(2)/(1+p(3));
                idx = idx + 2;
            end
        end
        % increment allpass section index
        idxSec = idxSec + 1;
    end
    
    %complex roots
    idx = 1;
    while idx <= length(rootsImag)
        if ((lfGetSameMag(rootsImag, idx, ctol) == 2) && (lfGetSameMag(rootsImag, idx+2, ctol) == 2))
            % create 4th order section (if both root pairs are purely imaginary)
            isImaginary = ((abs(real(rootsImag(idx))) < ctol) && (abs(real(rootsImag(idx+2))) < ctol));
            if (isImaginary)
                codes(1,idxSec) = 'D';
                p = poly([rootsImag(idx) rootsImag(idx+1) rootsImag(idx+2) rootsImag(idx+3)]);
                % TODO assertion for elements 2 and 4 of polynomial (!!!must be zero!!!)
                gamma(1,idxSec,1) = -p(5);
                gamma(2,idxSec,1) = -p(3)/(1+p(5));
                idx = idx + 4;
            else
                % create 2nd order section
                codes(1,idxSec) = 'd';
                p = poly([rootsImag(idx) rootsImag(idx+1)]);
                gamma(1,idxSec,1) = -p(3);
                gamma(2,idxSec,1) = -p(2)/(1+p(3));
                idx = idx + 2;
            end
        elseif (lfGetSameMag(rootsImag, idx, ctol) == 2)
            % create 2nd order section
            codes(1,idxSec) = 'd';
            p = poly([rootsImag(idx) rootsImag(idx+1)]);
            gamma(1,idxSec,1) = -p(3);
            gamma(2,idxSec,1) = -p(2)/(1+p(3));
            idx = idx + 2;
        else
            % create 4th order section
            codes(1,idxSec) = 'D';
            p = poly([rootsImag(idx) rootsImag(idx+1) rootsImag(idx+2) rootsImag(idx+3)]);
            % TODO assertion for elements 2 and 4 of polynomial (!!!must be zero!!!)
            gamma(1,idxSec,1) = -p(5);
            gamma(2,idxSec,1) = -p(3)/(1+p(5));
            idx = idx + 4;
        end
        % increment allpass section index
        idxSec = idxSec + 1;
    end
    
    % bottom allpass sections
    nSlices = idxSec - 2;
    orderBot = nSlices * 4;
    codes(2,1:nSlices) = char( 'D' * ones( 1,nSlices));
    gamma(:,1:nSlices,2) = zeros(2, nSlices);
    errStr = 'maximum number of AP sections not enough to cover bottom filter order';
    assert(orderBot >= numBot,'maximum number of AP sections not enough to cover bottom filter order')
    
    idxSec = 1;
    while (idxSec<=nSlices) && (numBot ~= orderBot)
        if ((idxSec == 1) && (rem(numBot,2) == 1))
            codes(2, idxSec) = 's';
            orderBot = orderBot - 3;
        else
            % check if we need to deduct more than 2
            codes(2, idxSec) = 'd';
            orderBot = orderBot - 2;
        end
        errStr = 'something went wrong in calculation of bottom row, removed too many delays!';
        assert(orderBot >= numBot, 'something went wrong in calculation of bottom row, removed too many delays!');
        idxSec = idxSec + 1;
    end
    errStr = 'bottom row AP calculation failed';
    assert(orderBot == numBot,'bottom row AP calculation failed')
    errStr = '';

catch
	if showMessages
		error(errStr);
	else
		LWDF = [];
		Hz   = [];
		Messages.warning = msgStr;
		Messages.error   = errStr;
	end
end

if isempty(errStr)
	LWDF.wdaCodes = codes;
	LWDF.gamma    = gamma;
	Hz = LWDF2Hz( LWDF );
	Messages.warning = msgStr;
	Messages.error   = errStr;

	if showMessages
		fprintf( msgStr );
	end

	if ( figNo ~= 0 )
        LWDF.gamma
        LWDF.wdaCodes
		showLWDF( LWDF, 'L', figNo );
	end
	if exist('plotOptionsString','var')
		if ( ischar(plotOptionsString) && ~isempty(plotOptionsString) )
			eval( ['plotHz(Hz,' plotOptionsString ');'] );
		else
			plotHz( Hz );
		end
	end
end


%==========================================================================================
%=========  LOCAL FUNCTIONS  ==============================================================
%==========================================================================================
function numRoots = lfGetSameMag( roots_gz, idx, ctol )
    absRoots = abs(roots_gz);
    if idx <= length(roots_gz)
        absIdx = abs(roots_gz(idx));
        numRoots = sum(abs(absRoots - absIdx) <= ctol);
    else
        numRoots = 0; % index out of range 
    end

function [rootsZero, rootsReal, rootsImag] = lfSortRoots( roots_gz, ctol )
    % sort roots in ascending order grouped in the following way
    % - roots at zero
    % - purely real roots + purely imaginary roots
    % - complex roots
    rootsSorted = sort(roots_gz, 'ascend'); % sort by ascending absolute value
    idxZero = ((abs(real(rootsSorted)) <  ctol) + (abs(imag(rootsSorted)) <  ctol)) > 1;
    idxReal = ((abs(real(rootsSorted)) >= ctol) + (abs(imag(rootsSorted)) <  ctol)) > 1;
    idxImag = ((abs(real(rootsSorted)) <  ctol) + (abs(imag(rootsSorted)) >= ctol)) > 1;
    idxComp = ((abs(real(rootsSorted)) >= ctol) + (abs(imag(rootsSorted)) >= ctol)) > 1;
    
    % assertions regarding multiplicity of roots
    %assert(sum(idxZero)<=1, 'more than one roots at 0!');
    assert(rem(sum(idxImag),2)==0, 'odd number of pure imaginary roots!');
    assert(rem(sum(idxComp),2)==0, 'odd number of complex roots!');
    
    rootsZero = rootsSorted(idxZero);
    rootsReal = rootsSorted(idxReal);
    rootsImag = [rootsSorted(idxImag) ; rootsSorted(idxComp)];

% EOF